@@ -25,15 +25,20 @@ from django.db import router, connections, transaction
 from django.db.models import (
     Manager, Model, Field, AutoField,
     ForeignKey, BigAutoField)
+from django.utils import version
 from django.db.models.fields.related import RelatedField
 from django.forms import modelform_factory
 from django.utils.functional import cached_property
 from django.utils.html import format_html_join, format_html
 from django.utils.safestring import mark_safe
 from django.utils.translation import gettext_lazy as _
-
 from djongo.exceptions import NotSupportedError, print_warn
 
+django_major = int(version.get_version().split('.')[0])
+
+if django_major >= 3:
+    from django.db.models.fields import AutoFieldMixin, AutoFieldMeta
+
 
 def make_mdl(model, model_dict):
     """
@@ -72,12 +77,32 @@ class DjongoManager(Manager):
                 .db_conn[self.model._meta.db_table])
 
 
-class FormlessField(Field):
+class MongoField(Field):
+    empty_strings_allowed = False
+
+
+class JSONField(MongoField):
+    def get_prep_value(self, value):
+        if not isinstance(value, (dict, list)):
+            raise ValueError(
+                f'Value: {value} must be of type dict/list'
+            )
+        return value
+
+    def to_python(self, value):
+        if not isinstance(value, (dict, list)):
+            raise ValueError(
+                f'Value: {value} stored in DB must be of type dict/list'
+                'Did you miss any Migrations?'
+            )
+        return value
+
+
+class ModelField(MongoField):
     """
     Allows for the inclusion of an instance of an abstract model as
     a field inside a document.
     """
-    empty_strings_allowed = False
     base_type = dict
 
     def __init__(self,
@@ -88,7 +113,7 @@ class FormlessField(Field):
         super().__init__(*args, **kwargs)
 
     def _validate_container(self):
-        for field in self.model_container._meta.get_fields():
+        for field in self.model_container._meta._get_fields(reverse=False):
             if isinstance(field, (AutoField,
                                   BigAutoField,
                                   RelatedField)):
@@ -180,19 +205,25 @@ class FormlessField(Field):
         kwargs['model_container'] = self.model_container
         return name, path, args, kwargs
 
-    def get_prep_value(self, value):
+    def get_db_prep_save(self, value, connection):
         if value is None:
             return None
 
         if not isinstance(value, self.base_type):
             raise ValueError(
                 f'Value: {value} must be an instance of {self.base_type}')
+        return self.get_prep_value(value)
+
+    def get_prep_value(self, value):
+        if (value is None or
+                not isinstance(value, self.base_type)):
+            return value
 
         processed_value = self._value_thru_fields('get_prep_value',
                                                   value)
         return processed_value
 
-    def from_db_value(self, value, expression, connection, context):
+    def from_db_value(self, value, *args):
         return self.to_python(value)
 
     def to_python(self, value):
@@ -215,7 +246,7 @@ class FormlessField(Field):
         return processed_value
 
 
-class MongoField(FormlessField):
+class FormedField(ModelField):
 
     def __init__(self,
                  model_container: typing.Type[Model],
@@ -250,7 +281,7 @@ class MongoField(FormlessField):
         return super().formfield(**defaults)
 
 
-class ArrayField(MongoField):
+class ArrayField(FormedField):
     """
     Implements an array of objects inside the document.
 
@@ -306,7 +337,7 @@ class ArrayField(MongoField):
         return processed_value
 
     def validate(self, value, model_instance, validate_parent=True):
-        super(FormlessField, self).validate(value, model_instance)
+        super(ModelField, self).validate(value, model_instance)
         for _dict in value:
             super().validate(_dict, model_instance, validate_parent=False)
 
@@ -459,7 +490,7 @@ class ArrayFormWidget(forms.Widget):
         return True
 
 
-class EmbeddedField(MongoField):
+class EmbeddedField(FormedField):
     pass
 
 
@@ -568,6 +599,7 @@ class EmbeddedFormWidget(forms.MultiWidget):
 
 class ObjectIdFieldMixin:
     description = _("ObjectId")
+    empty_strings_allowed = False
 
     def get_db_prep_value(self, value, connection, prepared=False):
         return self.to_python(value)
@@ -580,12 +612,23 @@ class ObjectIdFieldMixin:
     def get_internal_type(self):
         return "ObjectIdField"
 
+    def rel_db_type(self, connection):
+        return self.db_type(connection)
+
 
 class GenericObjectIdField(ObjectIdFieldMixin, Field):
-    empty_strings_allowed = False
+    pass
 
 
-class ObjectIdField(ObjectIdFieldMixin, AutoField):
+if django_major >= 3:
+    class _ObjectIdField(AutoFieldMixin, GenericObjectIdField, metaclass=AutoFieldMeta):
+        pass
+else:
+    class _ObjectIdField(ObjectIdFieldMixin, AutoField):
+        pass
+
+
+class ObjectIdField(_ObjectIdField):
     """
     For every document inserted into a collection MongoDB internally creates an field.
     The field can be referenced from within the Model as _id.
@@ -599,10 +642,6 @@ class ObjectIdField(ObjectIdFieldMixin, AutoField):
         id_field_args.update(kwargs)
         super().__init__(*args, **id_field_args)
 
-    def get_prep_value(self, value):
-        value = super(AutoField, self).get_prep_value(value)
-        return value
-
 
 class ArrayReferenceManagerMixin:
 
